/*
 ***********************************************************************
 ** md5.c -- the source code for MD5 routines                         **
 ** RSA Data Security, Inc. MD5 Message-Digest Algorithm              **
 ** Created: 2/17/90 RLR                                              **
 ** Revised: 1/91 SRD,AJ,BSK,JT Reference C ver., 7/10 constant corr. **
 ***********************************************************************
 */

/*
 ***********************************************************************
 ** Copyright (C) 1990, RSA Data Security, Inc. All rights reserved.  **
 **                                                                   **
 ** License to copy and use this software is granted provided that    **
 ** it is identified as the "RSA Data Security, Inc. MD5 Message-     **
 ** Digest Algorithm" in all material mentioning or referencing this  **
 ** software or this function.                                        **
 **                                                                   **
 ** License is also granted to make and use derivative works          **
 ** provided that such works are identified as "derived from the RSA  **
 ** Data Security, Inc. MD5 Message-Digest Algorithm" in all          **
 ** material mentioning or referencing the derived work.              **
 **                                                                   **
 ** RSA Data Security, Inc. makes no representations concerning       **
 ** either the merchantability of this software or the suitability    **
 ** of this software for any particular purpose.  It is provided "as  **
 ** is" without express or implied warranty of any kind.              **
 **                                                                   **
 ** These notices must be retained in any copies of any part of this  **
 ** documentation and/or software.                                    **
 ***********************************************************************
 */

#include "../securityLib.h"

#include <stdio.h>
#include "iframe/include/VC90Upgrade.h"

typedef struct
{
	IFRAME_DWORD32 i[2];                   								/* number of _bits_ handled mod 2^64 */
	IFRAME_DWORD32 buf[4];                                 /* scratch buffer */
	BYTE in[64];                    					    /* input buffer */
	BYTE digest[16];     													/* actual digest after MD5Final call */
} MD5_CTX;

static void Transform (void);
static void swap_bytes(LPBYTE CW);

/*
 ***********************************************************************
 **  Message-digest routines:                                         **
 **  To form the message digest for a message M                       **
 **    (1) Initialize a context buffer mdContext using MD5Init        **
 **    (2) Call MD5Update on mdContext and M                          **
 **    (3) Call MD5Final on mdContext                                 **
 **  The message digest is now in mdContext->digest[0...15]           **
 ***********************************************************************
 */

/* forward declaration */

/* F, G, H and I are basic MD5 functions */
#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define H(x, y, z) ((x) ^ (y) ^ (z))
#define I(x, y, z) ((y) ^ ((x) | (~z)))

/* ROTATE_LEFT rotates x left n bits */
#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4 */
/* Rotation is separate from addition to prevent recomputation */
#define FF(a, b, c, d, x, s, ac) \
	{(a) += F ((b), (c), (d)) + (x) + (IFRAME_DWORD32)(ac); \
	 (a) = ROTATE_LEFT ((a), (s)); \
	 (a) += (b); \
	}
#define GG(a, b, c, d, x, s, ac) \
	{(a) += G ((b), (c), (d)) + (x) + (IFRAME_DWORD32)(ac); \
	 (a) = ROTATE_LEFT ((a), (s)); \
	 (a) += (b); \
	}
#define HH(a, b, c, d, x, s, ac) \
	{(a) += H ((b), (c), (d)) + (x) + (IFRAME_DWORD32)(ac); \
	 (a) = ROTATE_LEFT ((a), (s)); \
	 (a) += (b); \
	}
#define II(a, b, c, d, x, s, ac) \
	{(a) += I ((b), (c), (d)) + (x) + (IFRAME_DWORD32)(ac); \
	 (a) = ROTATE_LEFT ((a), (s)); \
	 (a) += (b); \
	}


MD5_CTX mdContext;
IFRAME_DWORD32 in[16];
BYTE PADDING[64]=
{
	0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};


void GILSEC_API encryptString(char * item, unsigned char * outBuffer)
{
  BYTE in[1024];
  BYTE out[1024];

  memset(in,  0, 1024);
  memset(out, 0, 1024);

  // hier werden nur string mit einer maximalen lnge von 1024 byte akzeptiert.
  // todo: unicode verhalten (ein character mit 2 byte kodiert!)

  size_t i = strlen(item);
  memcpy(in, item, i > 1024 ? 1024 : i);

  MD5(in, i, out);

  // der md5 hash-value wird als 16 Byte Wert nach outBuffer kopiert
  memcpy(outBuffer, out, 16);

}

void GILSEC_API encryptStringAsHex(char * item, unsigned char * outBuffer)
{
  unsigned char buffer[16];
  unsigned char hexout[33];

  memset(buffer,  0, 16);
  memset(hexout,  0, 33);

  encryptString(item, buffer);

  for (int i=0; i<16; i++) {
    char c[3];
    SPRINTF_S(c, 3, "%02X", buffer[i]);

    hexout[i*2] = c[0];
    hexout[i*2+1] = c[1];
  }

  memcpy(outBuffer, hexout, 32);
  outBuffer[32] = 0x00;


}



/*******************************/
/* calculate the MD5 of a file */
/*******************************/

void GILSEC_API MD5(LPBYTE data,size_t bytes,LPBYTE hash)
{
	MD5Init ();
 	MD5Update(data,(WORD)bytes);
	MD5Final ();
	for(int i=0;i<16;i++)
	{
		hash[i]=mdContext.digest[i];
		mdContext.digest[i]=0;											// zero it for secrecy
	}
/*

	swap_bytes(&hash[0]);
	swap_bytes(&hash[4]);
	swap_bytes(&hash[8]);
	swap_bytes(&hash[12]);
*/
	
}


/***************************************/
/* Copy results of hash to user buffer */
/***************************************/

void GILSEC_API MD5GetHash(LPBYTE hash)
{
int i;

	for(i=0;i<16;i++)
	{
		hash[i]=mdContext.digest[i];
    mdContext.digest[i]=0;											/* zero it for secrecy */
  }
	swap_bytes(&hash[0]);
	swap_bytes(&hash[4]);
	swap_bytes(&hash[8]);
	swap_bytes(&hash[12]);
}


/* The routine MD5Init initializes the message-digest context
	 mdContext. All fields are set to zero.
 */
void GILSEC_API MD5Init (void)
{
	mdContext.i[0] = mdContext.i[1] = (IFRAME_DWORD32)0;

	/* Load magic initialization constants.
	 */

	mdContext.buf[0] = (IFRAME_DWORD32)0x67452301L;
	mdContext.buf[1] = (IFRAME_DWORD32)0xefcdab89L;
	mdContext.buf[2] = (IFRAME_DWORD32)0x98badcfeL;
	mdContext.buf[3] = (IFRAME_DWORD32)0x10325476L;
}

/* The routine MD5Update updates the message-digest context to
	 account for the presence of each of the characters inBuf[0..inLen-1]
	 in the message whose digest is being computed.
 */

void GILSEC_API MD5Update (LPBYTE inBuf,WORD inLen)
{
int mdi;
WORD i, ii;

/* compute number of bytes mod 64 */

	mdi = (int)((mdContext.i[0] >> 3) & 0x3F);

/* update number of bits */

	if ((mdContext.i[0] + ((IFRAME_DWORD32)inLen << 3)) < mdContext.i[0])
		mdContext.i[1]++;

	mdContext.i[0] += ((IFRAME_DWORD32)inLen << 3);
	mdContext.i[1] += ((IFRAME_DWORD32)inLen >> 29);

	while (inLen--)
	{

/* add new character to buffer, increment mdi */

		mdContext.in[mdi++] = *inBuf++;

/* transform if necessary */

    if (mdi == 0x40) {
			for (i = 0, ii = 0; i < 16; i++, ii += 4)
				in[i] = (((IFRAME_DWORD32)mdContext.in[ii+3]) << 24) |
								(((IFRAME_DWORD32)mdContext.in[ii+2]) << 16) |
								(((IFRAME_DWORD32)mdContext.in[ii+1]) << 8) |
								((IFRAME_DWORD32)mdContext.in[ii]);
			Transform ();
      mdi = 0;
    }
  }
}

/* The routine MD5Final terminates the message-digest computation and
	 ends with the desired message digest in mdContext.digest[0...15].
 */

void GILSEC_API MD5Final (void)
{
	int mdi;
	WORD i, ii;
	WORD padLen;

	/* save number of bits */

	in[14] = mdContext.i[0];
	in[15] = mdContext.i[1];

	/* compute number of bytes mod 64 */

	mdi = (int)((mdContext.i[0] >> 3) & 0x3F);

	/* pad out to 56 mod 64 */

	padLen = (WORD)((mdi < 56) ? (56 - mdi) : (120 - mdi));
	MD5Update ((LPBYTE )PADDING, padLen);

	/* append length in bits and transform */
	for (i = 0, ii = 0; i < 14; i++, ii += 4)
		in[i] = (((IFRAME_DWORD32)mdContext.in[ii+3]) << 24) |
						(((IFRAME_DWORD32)mdContext.in[ii+2]) << 16) |
						(((IFRAME_DWORD32)mdContext.in[ii+1]) << 8) |
						((IFRAME_DWORD32)mdContext.in[ii]);
	Transform ();

	/* store buffer in digest */
	for (i = 0, ii = 0; i < 4; i++, ii += 4) {
		mdContext.digest[ii] = (BYTE)(mdContext.buf[i] & 0xFF);
		mdContext.digest[ii+1] =
			(BYTE)((mdContext.buf[i] >> 8) & 0xFF);
		mdContext.digest[ii+2] =
			(BYTE)((mdContext.buf[i] >> 16) & 0xFF);
		mdContext.digest[ii+3] =
			(BYTE)((mdContext.buf[i] >> 24) & 0xFF);
	}
}

/* Basic MD5 step. Transforms buf based on in.
 */
static void Transform (void)
{
	IFRAME_DWORD32 a = mdContext.buf[0], b = mdContext.buf[1], c = mdContext.buf[2], d = mdContext.buf[3];

	/* Round 1 */
#define S11 7
#define S12 12
#define S13 17
#define S14 22
  FF ( a, b, c, d, in[ 0], S11, 3614090360UL); /* 1 */
  FF ( d, a, b, c, in[ 1], S12, 3905402710UL); /* 2 */
	FF ( c, d, a, b, in[ 2], S13,  606105819UL); /* 3 */
  FF ( b, c, d, a, in[ 3], S14, 3250441966UL); /* 4 */
  FF ( a, b, c, d, in[ 4], S11, 4118548399UL); /* 5 */
  FF ( d, a, b, c, in[ 5], S12, 1200080426UL); /* 6 */
	FF ( c, d, a, b, in[ 6], S13, 2821735955UL); /* 7 */
	FF ( b, c, d, a, in[ 7], S14, 4249261313UL); /* 8 */
  FF ( a, b, c, d, in[ 8], S11, 1770035416UL); /* 9 */
  FF ( d, a, b, c, in[ 9], S12, 2336552879UL); /* 10 */
  FF ( c, d, a, b, in[10], S13, 4294925233UL); /* 11 */
  FF ( b, c, d, a, in[11], S14, 2304563134UL); /* 12 */
  FF ( a, b, c, d, in[12], S11, 1804603682UL); /* 13 */
  FF ( d, a, b, c, in[13], S12, 4254626195UL); /* 14 */
  FF ( c, d, a, b, in[14], S13, 2792965006UL); /* 15 */
  FF ( b, c, d, a, in[15], S14, 1236535329UL); /* 16 */

	/* Round 2 */
#define S21 5
#define S22 9
#define S23 14
#define S24 20
  GG ( a, b, c, d, in[ 1], S21, 4129170786UL); /* 17 */
  GG ( d, a, b, c, in[ 6], S22, 3225465664UL); /* 18 */
  GG ( c, d, a, b, in[11], S23,  643717713UL); /* 19 */
  GG ( b, c, d, a, in[ 0], S24, 3921069994UL); /* 20 */
  GG ( a, b, c, d, in[ 5], S21, 3593408605UL); /* 21 */
  GG ( d, a, b, c, in[10], S22,   38016083UL); /* 22 */
	GG ( c, d, a, b, in[15], S23, 3634488961UL); /* 23 */
  GG ( b, c, d, a, in[ 4], S24, 3889429448UL); /* 24 */
  GG ( a, b, c, d, in[ 9], S21,  568446438UL); /* 25 */
  GG ( d, a, b, c, in[14], S22, 3275163606UL); /* 26 */
  GG ( c, d, a, b, in[ 3], S23, 4107603335UL); /* 27 */
  GG ( b, c, d, a, in[ 8], S24, 1163531501UL); /* 28 */
  GG ( a, b, c, d, in[13], S21, 2850285829UL); /* 29 */
  GG ( d, a, b, c, in[ 2], S22, 4243563512UL); /* 30 */
  GG ( c, d, a, b, in[ 7], S23, 1735328473UL); /* 31 */
  GG ( b, c, d, a, in[12], S24, 2368359562UL); /* 32 */

	/* Round 3 */
#define S31 4
#define S32 11
#define S33 16
#define S34 23
  HH ( a, b, c, d, in[ 5], S31, 4294588738UL); /* 33 */
  HH ( d, a, b, c, in[ 8], S32, 2272392833UL); /* 34 */
  HH ( c, d, a, b, in[11], S33, 1839030562UL); /* 35 */
  HH ( b, c, d, a, in[14], S34, 4259657740UL); /* 36 */
  HH ( a, b, c, d, in[ 1], S31, 2763975236UL); /* 37 */
  HH ( d, a, b, c, in[ 4], S32, 1272893353UL); /* 38 */
	HH ( c, d, a, b, in[ 7], S33, 4139469664UL); /* 39 */
  HH ( b, c, d, a, in[10], S34, 3200236656UL); /* 40 */
  HH ( a, b, c, d, in[13], S31,  681279174UL); /* 41 */
  HH ( d, a, b, c, in[ 0], S32, 3936430074UL); /* 42 */
  HH ( c, d, a, b, in[ 3], S33, 3572445317UL); /* 43 */
  HH ( b, c, d, a, in[ 6], S34,   76029189UL); /* 44 */
	HH ( a, b, c, d, in[ 9], S31, 3654602809UL); /* 45 */
	HH ( d, a, b, c, in[12], S32, 3873151461UL); /* 46 */
	HH ( c, d, a, b, in[15], S33,  530742520UL); /* 47 */
  HH ( b, c, d, a, in[ 2], S34, 3299628645UL); /* 48 */

  /* Round 4 */
#define S41 6
#define S42 10
#define S43 15
#define S44 21
  II ( a, b, c, d, in[ 0], S41, 4096336452UL); /* 49 */
  II ( d, a, b, c, in[ 7], S42, 1126891415UL); /* 50 */
  II ( c, d, a, b, in[14], S43, 2878612391UL); /* 51 */
	II ( b, c, d, a, in[ 5], S44, 4237533241UL); /* 52 */
  II ( a, b, c, d, in[12], S41, 1700485571UL); /* 53 */
  II ( d, a, b, c, in[ 3], S42, 2399980690UL); /* 54 */
	II ( c, d, a, b, in[10], S43, 4293915773UL); /* 55 */
  II ( b, c, d, a, in[ 1], S44, 2240044497UL); /* 56 */
  II ( a, b, c, d, in[ 8], S41, 1873313359UL); /* 57 */
  II ( d, a, b, c, in[15], S42, 4264355552UL); /* 58 */
  II ( c, d, a, b, in[ 6], S43, 2734768916UL); /* 59 */
  II ( b, c, d, a, in[13], S44, 1309151649UL); /* 60 */
  II ( a, b, c, d, in[ 4], S41, 4149444226UL); /* 61 */
  II ( d, a, b, c, in[11], S42, 3174756917UL); /* 62 */
	II ( c, d, a, b, in[ 2], S43,  718787259UL); /* 63 */
  II ( b, c, d, a, in[ 9], S44, 3951481745UL); /* 64 */

	mdContext.buf[0] += a;
	mdContext.buf[1] += b;
	mdContext.buf[2] += c;
	mdContext.buf[3] += d;
}


/*******************************/
/* Swap byte order in a word32 */
/*******************************/

static void swap_bytes(LPBYTE CW)
{
BYTE temp;

	temp=CW[0];
	CW[0]=CW[3];
	CW[3]=temp;
	temp=CW[1];
	CW[1]=CW[2];
	CW[2]=temp;
}

