/*
 * Karn (MDC) encryption
 * Based on Phil Karn, sci.crypt, 13 Feb 1992
 * See also his comments from sci.crypt, 23 Mar 1992.
 * The method is a variant of that described in
 * Zheng, Matsumoto and Imai, Crypto 89.
 * See also, "A New Class of Cryptosystems Based on
 * Interconnection Networks" by
 * michaelp@terpsichore.informatic.rwth-aachen.de
 *
 * A method for turning a hash function, here MD5, into a fast
 * secret-key encryption.
 *
 * This does triple hashing with nondistinct keys.
 *
 * adapted to a Windows DLL by Andy Brown 1994
                MD5 hash algorithm DLL by Andy Brown 1994
         -----------------------------------------
 */

#include "../securityLib.h"

/**
 * This is the basic encryption function that encrypts the plaintext
 * in `in' and leaves the ciphertext in `out'
 *
 * @param in      FAR pointer to a 32 byte block to be encrypted
 * @param out     FAR pointer to a 32 byte block to receive the ciphertext
 * @param key     FAR pointer to a 96 byte key
 * 
 */
GILSEC_API void mdc_encrypt(LPBYTE in, LPBYTE out, LPBYTE key);

/**
 * This is the inverse of mdc_encrypt. The ciphertext in `in' is
 * decrypted, and the plaintext left in `out'
 *
 * @param in      FAR pointer to 32 bytes of ciphertext to decrypt
 * @param out     FAR pointer to a 32 byte buffer to receive the plaintext
 * @param key     FAR pointer to a 96 byte key
 * 
 */
GILSEC_API void mdc_decrypt(LPBYTE in, LPBYTE out, LPBYTE key);
